@@ -5,24 +5,89 @@ import os
 from ceph_deploy import hosts
 from ceph_deploy.cliutil import priority
 from ceph_deploy.lib import remoto
-
+from ceph_deploy.util.constants import default_components
+from ceph_deploy.util.paths import gpg
 
 LOG = logging.getLogger(__name__)
 
 
-def install(args):
-    if args.repo:
-        return install_repo(args)
+def sanitize_args(args):
+    """
+    args may need a bunch of logic to set proper defaults that argparse is
+    not well suited for.
+    """
+    if args.release is None:
+        args.release = 'hammer'
+        args.default_release = True
 
     # XXX This whole dance is because --stable is getting deprecated
     if args.stable is not None:
         LOG.warning('the --stable flag is deprecated, use --release instead')
         args.release = args.stable
+    # XXX Tango ends here.
+
+    return args
+
+
+def detect_components(args, distro):
+    """
+    Since the package split, now there are various different Ceph components to
+    install like:
+
+    * ceph
+    * ceph-mon
+    * ceph-osd
+    * ceph-mds
+
+    This helper function should parse the args that may contain specifics about
+    these flags and return the default if none are passed in (which is, install
+    everything)
+    """
+    # the flag that prevents all logic here is the `--repo` flag which is used
+    # when no packages should be installed, just the repo files, so check for
+    # that here and return an empty list (which is equivalent to say 'no
+    # packages should be installed')
+    if args.repo:
+        return []
+
+    flags = {
+        'install_osd': 'ceph-osd',
+        'install_rgw': 'ceph-radosgw',
+        'install_mds': 'ceph-mds',
+        'install_mon': 'ceph-mon',
+        'install_common': 'ceph-common',
+    }
+
+    if distro.is_rpm:
+        defaults = default_components.rpm
+    else:
+        defaults = default_components.deb
+        # different naming convention for deb than rpm for radosgw
+        flags['install_rgw'] = 'radosgw'
+
+    if args.install_all:
+        return defaults
+    else:
+        components = []
+        for k, v in flags.items():
+            if getattr(args, k, False):
+                components.append(v)
+        # if we have some components selected from flags then return that,
+        # otherwise return defaults because no flags and no `--repo` means we
+        # should get all of them by default
+        return components or defaults
+
+
+def install(args):
+    args = sanitize_args(args)
+
+    if args.repo:
+        return install_repo(args)
+
     if args.version_kind == 'stable':
         version = args.release
     else:
         version = getattr(args, args.version_kind)
-    # XXX Tango ends here.
 
     version_str = args.version_kind
 
@@ -37,7 +102,17 @@ def install(args):
 
     for hostname in args.host:
         LOG.debug('Detecting platform for host %s ...', hostname)
-        distro = hosts.get(hostname, username=args.username)
+        distro = hosts.get(
+            hostname,
+            username=args.username,
+            # XXX this should get removed once ceph packages are split for
+            # upstream. If default_release is True, it means that the user is
+            # trying to install on a RHEL machine and should expect to get RHEL
+            # packages. Otherwise, it will need to specify either a specific
+            # version, or repo, or a development branch. Other distro users
+            # should not see any differences.
+            use_rhceph=args.default_release,
+            )
         LOG.info(
             'Distro info: %s %s %s',
             distro.name,
@@ -45,6 +120,7 @@ def install(args):
             distro.codename
         )
 
+        components = detect_components(args, distro)
         if distro.init == 'sysvinit' and args.cluster != 'ceph':
             LOG.error('refusing to install on host: %s, with custom cluster name: %s' % (
                     hostname,
@@ -55,14 +131,14 @@ def install(args):
             continue
 
         rlogger = logging.getLogger(hostname)
-        rlogger.info('installing ceph on %s' % hostname)
+        rlogger.info('installing Ceph on %s' % hostname)
 
         cd_conf = getattr(args, 'cd_conf', None)
 
         # custom repo arguments
         repo_url = os.environ.get('CEPH_DEPLOY_REPO_URL') or args.repo_url
         gpg_url = os.environ.get('CEPH_DEPLOY_GPG_URL') or args.gpg_url
-        gpg_fallback = 'https://ceph.com/git/?p=ceph.git;a=blob_plain;f=keys/release.asc'
+        gpg_fallback = gpg.url('release')
 
         if gpg_url is None and repo_url:
             LOG.warning('--gpg-url was not used, will fallback')
@@ -90,7 +166,8 @@ def install(args):
                 distro,
                 repo_url,
                 gpg_url,
-                args.adjust_repos
+                args.adjust_repos,
+                components=components,
             )
 
         # Detect and install custom repos here if needed
@@ -103,7 +180,8 @@ def install(args):
                 distro,
                 args.version_kind,
                 version,
-                args.adjust_repos
+                args.adjust_repos,
+                components=components,
             )
 
         # Check the ceph version we just installed
@@ -138,6 +216,7 @@ def custom_repo(distro, args, cd_conf, rlogger, install_ceph=None):
     used.
     """
     default_repo = cd_conf.get_default_repo()
+    components = detect_components(args, distro)
     if args.release in cd_conf.get_repos():
         LOG.info('will use repository from conf: %s' % args.release)
         default_repo = args.release
@@ -160,6 +239,7 @@ def custom_repo(distro, args, cd_conf, rlogger, install_ceph=None):
                 default_repo,
                 options.pop('baseurl'),
                 options.pop('gpgkey'),
+                components=components,
                 **options
             )
         except KeyError as err:
@@ -174,6 +254,7 @@ def custom_repo(distro, args, cd_conf, rlogger, install_ceph=None):
                     xrepo,
                     options.pop('baseurl'),
                     options.pop('gpgkey'),
+                    components=components,
                     **options
                 )
             except KeyError as err:
@@ -183,13 +264,23 @@ def custom_repo(distro, args, cd_conf, rlogger, install_ceph=None):
 def install_repo(args):
     """
     For a user that only wants to install the repository only (and avoid
-    installing ceph and its dependencies).
+    installing Ceph and its dependencies).
     """
     cd_conf = getattr(args, 'cd_conf', None)
 
     for hostname in args.host:
         LOG.debug('Detecting platform for host %s ...', hostname)
-        distro = hosts.get(hostname, username=args.username)
+        distro = hosts.get(
+            hostname,
+            username=args.username,
+            # XXX this should get removed once Ceph packages are split for
+            # upstream. If default_release is True, it means that the user is
+            # trying to install on a RHEL machine and should expect to get RHEL
+            # packages. Otherwise, it will need to specify either a specific
+            # version, or repo, or a development branch. Other distro users should
+            # not see any differences.
+            use_rhceph=args.default_release,
+        )
         rlogger = logging.getLogger(hostname)
 
         LOG.info(
@@ -199,7 +290,7 @@ def install_repo(args):
             distro.codename
         )
 
-        return custom_repo(distro, args, cd_conf, rlogger, install_ceph=False)
+        custom_repo(distro, args, cd_conf, rlogger, install_ceph=False)
 
 
 def uninstall(args):
@@ -214,11 +305,14 @@ def uninstall(args):
     for hostname in args.host:
         LOG.debug('Detecting platform for host %s ...', hostname)
 
-        distro = hosts.get(hostname, username=args.username)
+        distro = hosts.get(
+            hostname,
+            username=args.username,
+            use_rhceph=True)
         LOG.info('Distro info: %s %s %s', distro.name, distro.release, distro.codename)
         rlogger = logging.getLogger(hostname)
-        rlogger.info('uninstalling ceph on %s' % hostname)
-        distro.uninstall(distro.conn)
+        rlogger.info('uninstalling Ceph on %s' % hostname)
+        distro.uninstall(distro)
         distro.conn.exit()
 
 
@@ -235,11 +329,15 @@ def purge(args):
     for hostname in args.host:
         LOG.debug('Detecting platform for host %s ...', hostname)
 
-        distro = hosts.get(hostname, username=args.username)
+        distro = hosts.get(
+            hostname,
+            username=args.username,
+            use_rhceph=True
+        )
         LOG.info('Distro info: %s %s %s', distro.name, distro.release, distro.codename)
         rlogger = logging.getLogger(hostname)
         rlogger.info('purging host ... %s' % hostname)
-        distro.uninstall(distro.conn, purge=True)
+        distro.uninstall(distro, purge=True)
         distro.conn.exit()
 
 
@@ -259,8 +357,8 @@ def purgedata(args):
         distro.conn.exit()
 
     if installed_hosts:
-        LOG.error("ceph is still installed on: %s", installed_hosts)
-        raise RuntimeError("refusing to purge data while ceph is still installed")
+        LOG.error("Ceph is still installed on: %s", installed_hosts)
+        raise RuntimeError("refusing to purge data while Ceph is still installed")
 
     for hostname in args.host:
         distro = hosts.get(hostname, username=args.username)
@@ -381,34 +479,80 @@ def make(parser):
                 or tag (default: %(default)s)',
     )
 
-    version.add_argument(
+    version.set_defaults(
+        stable=None,  # XXX deprecated in favor of release
+        release=None,  # Set the default release in sanitize_args()
+        dev='master',
+        version_kind='stable',
+    )
+
+    parser.add_argument(
+        '--mon',
+        dest='install_mon',
+        action='store_true',
+        help='install the mon component only',
+    )
+
+    parser.add_argument(
+        '--mds',
+        dest='install_mds',
+        action='store_true',
+        help='install the mds component only',
+    )
+
+    parser.add_argument(
+        '--rgw',
+        dest='install_rgw',
+        action='store_true',
+        help='install the rgw component only',
+    )
+
+    parser.add_argument(
+        '--osd',
+        dest='install_osd',
+        action='store_true',
+        help='install the osd component only',
+    )
+
+    parser.add_argument(
+        '--cli', '--common',
+        dest='install_common',
+        action='store_true',
+        help='install the common component only',
+    )
+
+    parser.add_argument(
+        '--all',
+        dest='install_all',
+        action='store_true',
+        help='install all Ceph components (e.g. mon,osd,mds,rgw). This is the default',
+    )
+
+    repo = parser.add_mutually_exclusive_group()
+
+    repo.add_argument(
         '--adjust-repos',
         dest='adjust_repos',
         action='store_true',
         help='install packages modifying source repos',
     )
 
-    version.add_argument(
+    repo.add_argument(
         '--no-adjust-repos',
         dest='adjust_repos',
         action='store_false',
         help='install packages without modifying source repos',
     )
 
-    version.set_defaults(
-        func=install,
-        stable=None,  # XXX deprecated in favor of release
-        release='giant',
-        dev='master',
-        version_kind='stable',
-        adjust_repos=True,
-    )
-
-    parser.add_argument(
+    repo.add_argument(
         '--repo',
         action='store_true',
         help='install repo files only (skips package installation)',
-        )
+    )
+
+    repo.set_defaults(
+        adjust_repos=True,
+    )
 
     parser.add_argument(
         'host',
@@ -429,7 +573,7 @@ def make(parser):
         '--repo-url',
         nargs='?',
         dest='repo_url',
-        help='specify a repo URL that mirrors/contains ceph packages',
+        help='specify a repo URL that mirrors/contains Ceph packages',
     )
 
     parser.add_argument(
